@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  * 
  **********************************************************************
 
@@ -32,6 +32,7 @@
 #define GOT_CMDPARSE_H
 
 #include "srcparams.h"
+#include "addressing.h"
 
 typedef enum {
   CPS_Success,
@@ -47,7 +48,7 @@ typedef enum {
 } CPS_Status;
 
 typedef struct {
-  unsigned long ip_addr;
+  IPAddr ip_addr;
   unsigned short port;
   SourceParameters params;
 } CPS_NTP_Source;
