@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  * 
  **********************************************************************
 
@@ -31,12 +31,15 @@
 #ifndef GOT_CONF_H
 #define GOT_CONF_H
 
+#include "addressing.h"
+
 extern char *CNF_GetRtcDevice(void);
 
 extern void CNF_ReadFile(const char *filename);
 
 extern void CNF_AddSources(void);
 extern void CNF_AddBroadcasts(void);
+extern void CNF_AddRefclocks(void);
 
 extern void CNF_ProcessInitStepSlew(void (*after_hook)(void *), void *anything);
 
@@ -49,6 +52,7 @@ extern int CNF_GetLogMeasurements(void);
 extern int CNF_GetLogStatistics(void);
 extern int CNF_GetLogTracking(void);
 extern int CNF_GetLogRtc(void);
+extern int CNF_GetLogRefclocks(void);
 extern char *CNF_GetKeysFile(void);
 extern char *CNF_GetRtcFile(void);
 extern unsigned long CNF_GetCommandKey(void);
@@ -56,11 +60,13 @@ extern int CNF_GetDumpOnExit(void);
 extern int CNF_GetManualEnabled(void);
 extern int CNF_GetCommandPort(void);
 extern int CNF_GetRTCOnUTC(void);
+extern void CNF_GetMakeStep(int *limit, double *threshold);
 extern void CNF_GetLogChange(int *enabled, double *threshold);
 extern void CNF_GetMailOnChange(int *enabled, double *threshold, char **user);
 extern int CNF_GetNoClientLog(void);
-extern void CNF_GetBindAddress(unsigned long *addr);
-extern void CNF_GetBindCommandAddress(unsigned long *addr);
+extern unsigned long CNF_GetClientLogLimit(void);
+extern void CNF_GetBindAddress(int family, IPAddr *addr);
+extern void CNF_GetBindCommandAddress(int family, IPAddr *addr);
 extern char *CNF_GetPidFile(void);
 extern void CNF_GetLinuxHz(int *set, int *hz);
 extern void CNF_GetLinuxFreqScale(int *set, double *freq_scale);
@@ -71,4 +77,7 @@ extern int CNF_AllowLocalReference(int *stratum);
 
 extern void CNF_SetupAccessRestrictions(void);
 
+extern int CNF_GetSchedPriority(void);
+extern int CNF_GetLockMemory(void);
+
 #endif /* GOT_CONF_H */
