@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/local.c,v 1.15 1999/09/21 21:03:26 richard Exp $
+  $Header: /cvs/src/chrony/local.c,v 1.21 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -40,7 +53,7 @@ static lcl_SetFrequencyDriver drv_set_freq;
 static lcl_AccrueOffsetDriver drv_accrue_offset;
 static lcl_ApplyStepOffsetDriver drv_apply_step_offset;
 static lcl_OffsetCorrectionDriver drv_offset_convert;
-static lcl_ImmediateStepDriver drv_immediate_step;
+static lcl_SetLeapDriver drv_set_leap;
 
 /* ================================================== */
 
@@ -116,7 +129,7 @@ calculate_sys_precision(void)
     best_dusec *= 2;
   }
 
-  precision_quantum = 1.0 / (double)(1<<precision_log);
+  precision_quantum = 1.0 / (double)(1<<(-precision_log));
 
   return;
 }
@@ -484,8 +497,8 @@ LCL_AccumulateFrequencyAndOffset(double dfreq, double doffset)
   current_freq_ppm = (1.0 - dfreq) * old_freq_ppm +
     (1.0e6 * dfreq);
 
-#if 0
-  LOG(LOGS_INFO, LOGF_Local, "old_freq=%.3fppm new_freq=%.3fppm offset=%.6fsec\n",
+#ifdef TRACEON
+  LOG(LOGS_INFO, LOGF_Local, "old_freq=%.3fppm new_freq=%.3fppm offset=%.6fsec",
       old_freq_ppm, current_freq_ppm, doffset);
 #endif
 
@@ -522,19 +535,19 @@ lcl_RegisterSystemDrivers(lcl_ReadFrequencyDriver read_freq,
                           lcl_AccrueOffsetDriver accrue_offset,
                           lcl_ApplyStepOffsetDriver apply_step_offset,
                           lcl_OffsetCorrectionDriver offset_convert,
-                          lcl_ImmediateStepDriver immediate_step)
+                          lcl_SetLeapDriver set_leap)
 {
   drv_read_freq = read_freq;
   drv_set_freq = set_freq;
   drv_accrue_offset = accrue_offset;
   drv_apply_step_offset = apply_step_offset;
   drv_offset_convert = offset_convert;
-  drv_immediate_step = immediate_step;
+  drv_set_leap = set_leap;
 
   current_freq_ppm = (*drv_read_freq)();
 
-#if 0
-  LOG(LOGS_INFO, LOGF_Local, "Local freq=%.3fppm\n", current_freq_ppm);
+#ifdef TRACEON
+  LOG(LOGS_INFO, LOGF_Local, "Local freq=%.3fppm", current_freq_ppm);
 #endif
 
   return;
@@ -542,18 +555,39 @@ lcl_RegisterSystemDrivers(lcl_ReadFrequencyDriver read_freq,
 
 /* ================================================== */
 /* Look at the current difference between the system time and the NTP
-   time, and make a step to cancel it. */
+   time, and make a step to cancel it if it's larger than the threshold. */
 
 int
-LCL_MakeStep(void)
+LCL_MakeStep(double threshold)
 {
-  if (drv_immediate_step) {
-    (drv_immediate_step)();
-    LOG(LOGS_INFO, LOGF_Local, "Made step to system time to apply remaining slew");
-    return 1;
-  }
+  struct timeval raw;
+  double correction;
 
-  return 0;
+  LCL_ReadRawTime(&raw);
+  correction = LCL_GetOffsetCorrection(&raw);
+
+  if (fabs(correction) <= threshold)
+    return 0;
+
+  /* Cancel remaining slew and make the step */
+  LCL_AccumulateOffset(correction);
+  LCL_ApplyStepOffset(-correction);
+
+  LOG(LOGS_WARN, LOGF_Local, "System clock was stepped by %.3f seconds", correction);
+
+  return 1;
+}
+
+/* ================================================== */
+
+void
+LCL_SetLeap(int leap)
+{
+  if (drv_set_leap) {
+    (drv_set_leap)(leap);
+  }
+
+  return;
 }
 
 /* ================================================== */
