@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  * 
  **********************************************************************
 
@@ -287,7 +287,7 @@ MNL_ReportSamples(RPT_ManualSamplesReport *report, int max, int *n)
   }
 
   for (i=0; i<n_samples && i<max; i++) {
-    report[i].when = samples[i].when.tv_sec;
+    report[i].when = samples[i].when;
     report[i].slewed_offset = samples[i].offset;
     report[i].orig_offset = samples[i].orig_offset;
     report[i].residual = samples[i].residual;
