@@ -1,14 +1,28 @@
 /*
-  $Header: /cvs/src/chrony/reference.c,v 1.34 1999/09/21 21:04:12 richard Exp $
+  $Header: /cvs/src/chrony/reference.c,v 1.42 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * Copyright (C) Miroslav Lichvar  2009
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -31,8 +45,10 @@ static int are_we_synchronised;
 static int enable_local_stratum;
 static int local_stratum;
 static NTP_Leap our_leap_status;
+static int our_leap_sec;
 static int our_stratum;
 static unsigned long our_ref_id;
+static IPAddr our_ref_ip;
 struct timeval our_ref_time; /* Stored relative to reference, NOT local time */
 static double our_offset;
 static double our_skew;
@@ -45,6 +61,10 @@ static double max_update_skew;
 /* Flag indicating that we are initialised */
 static int initialised = 0;
 
+/* Threshold and update limit for stepping clock */
+static int make_step_limit;
+static double make_step_threshold;
+
 /* Flag and threshold for logging clock changes to syslog */
 static int do_log_change;
 static double log_change_threshold;
@@ -71,9 +91,6 @@ static unsigned long logwrites = 0;
 
 /* ================================================== */
 
-/* Day number of 1 Jan 1970 */
-#define MJD_1970 40587
-
 /* Reference ID supplied when we are locally referenced */
 #define LOCAL_REFERENCE_ID 0x7f7f0101UL
 
@@ -89,7 +106,8 @@ REF_Initialise(void)
   double our_frequency_ppm;
 
   are_we_synchronised = 0;
-  our_leap_status = LEAP_Normal;
+  our_leap_status = LEAP_Unsynchronised;
+  our_leap_sec = 0;
   initialised = 1;
   our_root_dispersion = 1.0;
   our_root_delay = 1.0;
@@ -108,16 +126,16 @@ REF_Initialise(void)
           our_frequency_ppm = file_freq_ppm;
           our_skew = 1.0e-6 * file_skew_ppm;
         } else {
-          LOG(LOGS_WARN, LOGF_Reference, "Could not parse valid frequency and skew from driftfile %s\n",
+          LOG(LOGS_WARN, LOGF_Reference, "Could not parse valid frequency and skew from driftfile %s",
               drift_file);
         }
       } else {
-        LOG(LOGS_WARN, LOGF_Reference, "Could not read valid frequency and skew from driftfile %s\n",
+        LOG(LOGS_WARN, LOGF_Reference, "Could not read valid frequency and skew from driftfile %s",
             drift_file);
       }
       fclose(in);
     } else {
-      LOG(LOGS_WARN, LOGF_Reference, "Could not open driftfile %s for reading\n",
+      LOG(LOGS_WARN, LOGF_Reference, "Could not open driftfile %s for reading",
           drift_file);
     }
 
@@ -129,7 +147,7 @@ REF_Initialise(void)
   if (CNF_GetLogTracking()) {
     direc = CNF_GetLogDir();
     if (!mkdir_and_parents(direc)) {
-      LOG(LOGS_ERR, LOGF_Reference, "Could not create directory %s\n", direc);
+      LOG(LOGS_ERR, LOGF_Reference, "Could not create directory %s", direc);
       logfile = NULL;
     } else {
       logfilename = MallocArray(char, 2 + strlen(direc) + strlen(TRACKING_LOG));
@@ -138,7 +156,7 @@ REF_Initialise(void)
       strcat(logfilename, TRACKING_LOG);
       logfile = fopen(logfilename, "a");
       if (!logfile) {
-        LOG(LOGS_WARN, LOGF_Reference, "Couldn't open logfile %s for update\n", logfilename);
+        LOG(LOGS_WARN, LOGF_Reference, "Couldn't open logfile %s for update", logfilename);
       }
     }
   }
@@ -147,6 +165,7 @@ REF_Initialise(void)
 
   enable_local_stratum = CNF_AllowLocalReference(&local_stratum);
 
+  CNF_GetMakeStep(&make_step_limit, &make_step_threshold);
   CNF_GetLogChange(&do_log_change, &log_change_threshold);
   CNF_GetMailOnChange(&do_mail_change, &mail_change_threshold, &mail_change_user);
 
@@ -163,6 +182,10 @@ REF_Initialise(void)
 void
 REF_Finalise(void)
 {
+  if (our_leap_sec) {
+    LCL_SetLeap(0);
+  }
+
   if (logfile) {
     fclose(logfile);
   }
@@ -212,7 +235,7 @@ update_drift_file(double freq_ppm, double skew)
   out = fopen(temp_drift_file, "w");
   if (!out) {
     Free(temp_drift_file);
-    LOG(LOGS_WARN, LOGF_Reference, "Could not open temporary driftfile %s.tmp for writing\n",
+    LOG(LOGS_WARN, LOGF_Reference, "Could not open temporary driftfile %s.tmp for writing",
         drift_file);
     return;
   }
@@ -225,7 +248,9 @@ update_drift_file(double freq_ppm, double skew)
   /* Clone the file attributes from the existing file if there is one. */
 
   if (!stat(drift_file,&buf)) {
-    chown(temp_drift_file,buf.st_uid,buf.st_gid);
+    if (chown(temp_drift_file,buf.st_uid,buf.st_gid)) {
+      LOG(LOGS_WARN, LOGF_Reference, "Could not change ownership of temporary driftfile %s.tmp", drift_file);
+    }
     chmod(temp_drift_file,buf.st_mode&0777);
   }
 
@@ -234,7 +259,7 @@ update_drift_file(double freq_ppm, double skew)
   if (rename(temp_drift_file,drift_file)) {
     unlink(temp_drift_file);
     Free(temp_drift_file);
-    LOG(LOGS_WARN, LOGF_Reference, "Could not replace old driftfile %s with new one %s.tmp (%d)\n",
+    LOG(LOGS_WARN, LOGF_Reference, "Could not replace old driftfile %s with new one %s.tmp (%d)",
         drift_file,drift_file);
     return;
   }
@@ -244,12 +269,15 @@ update_drift_file(double freq_ppm, double skew)
 
 /* ================================================== */
 
+#define BUFLEN 255
+#define S_MAX_USER_LEN "128"
+
 static void
 maybe_log_offset(double offset)
 {
   double abs_offset;
   FILE *p;
-  char buffer[256], host[256];
+  char buffer[BUFLEN], host[BUFLEN];
   time_t now;
   struct tm stm;
 
@@ -258,13 +286,13 @@ maybe_log_offset(double offset)
   if (do_log_change &&
       (abs_offset > log_change_threshold)) {
     LOG(LOGS_WARN, LOGF_Reference,
-        "System clock changed by %.6f seconds",
+        "System clock wrong by %.6f seconds, adjustment started",
         -offset);
   }
 
   if (do_mail_change &&
       (abs_offset > mail_change_threshold)) {
-    sprintf(buffer, "%s %s", MAIL_PROGRAM, mail_change_user);
+    snprintf(buffer, sizeof(buffer), "%s %." S_MAX_USER_LEN "s", MAIL_PROGRAM, mail_change_user);
     p = popen(buffer, "w");
     if (p) {
       if (gethostname(host, sizeof(host)) < 0) {
@@ -280,7 +308,7 @@ maybe_log_offset(double offset)
          positive change to it to bring it into line, hence the
          negation of 'offset' in the next statement (and earlier) */
       fprintf(p,
-              "\n\nchronyd applied an adjustment of %.3f seconds to it,\n"
+              "\n\nchronyd started to apply an adjustment of %.3f seconds to it,\n"
               "  which exceeded the reporting threshold of %.3f seconds\n\n",
               -offset, mail_change_threshold);
       pclose(p);
@@ -295,11 +323,83 @@ maybe_log_offset(double offset)
 
 /* ================================================== */
 
+static void
+maybe_make_step()
+{
+  if (make_step_limit == 0) {
+    return;
+  } else if (make_step_limit > 0) {
+    make_step_limit--;
+  }
+  LCL_MakeStep(make_step_threshold);
+}
+
+/* ================================================== */
+
+static void
+update_leap_status(NTP_Leap leap)
+{
+  time_t now;
+  struct tm stm;
+  int leap_sec;
+
+  leap_sec = 0;
+
+  if (leap == LEAP_InsertSecond || leap == LEAP_DeleteSecond) {
+    /* Insert/delete leap second only on June 30 or December 31
+       and in other months ignore the leap status completely */
+
+    now = time(NULL);
+    stm = *gmtime(&now);
+
+    if (stm.tm_mon != 5 && stm.tm_mon != 11) {
+      leap = LEAP_Normal;
+    } else if ((stm.tm_mon == 5 && stm.tm_mday == 30) ||
+        (stm.tm_mon == 11 && stm.tm_mday == 31)) {
+      if (leap == LEAP_InsertSecond) {
+        leap_sec = 1;
+      } else {
+        leap_sec = -1;
+      }
+    }
+  }
+  
+  if (leap_sec != our_leap_sec) {
+    LCL_SetLeap(leap_sec);
+    our_leap_sec = leap_sec;
+  }
+
+  our_leap_status = leap;
+}
+
+/* ================================================== */
+
+static void
+write_log(struct timeval *ref_time, char *ref, int stratum, double freq, double skew, double offset)
+{
+  if (logfile) {
+
+    if (((logwrites++) % 32) == 0) {
+      fprintf(logfile,
+              "=======================================================================\n"
+              "   Date (UTC) Time     IP Address   St   Freq ppm   Skew ppm     Offset\n"
+              "=======================================================================\n");
+    }
+          
+    fprintf(logfile, "%s %-15s %2d %10.3f %10.3f %10.3e\n",
+            UTI_TimeToLogForm(ref_time->tv_sec), ref, stratum, freq, skew, offset);
+    
+    fflush(logfile);
+  }
+}
+
+/* ================================================== */
 
 void
 REF_SetReference(int stratum,
                  NTP_Leap leap,
                  unsigned long ref_id,
+                 IPAddr *ref_ip,
                  struct timeval *ref_time,
                  double offset,
                  double frequency,
@@ -319,6 +419,12 @@ REF_SetReference(int stratum,
 
   assert(initialised);
 
+  /* Avoid getting NaNs */
+  if (skew == 0.0)
+    skew = 1e-10;
+  if (our_skew == 0.0)
+    our_skew = 1e-10;
+
   /* If we get a serious rounding error in the source stats regression
      processing, there is a remote chance that the skew argument is a
      'not a number'.  If such a quantity gets propagated into the
@@ -340,13 +446,18 @@ REF_SetReference(int stratum,
 
   are_we_synchronised = 1;
   our_stratum = stratum + 1;
-  our_leap_status = leap;
   our_ref_id = ref_id;
+  if (ref_ip)
+    our_ref_ip = *ref_ip;
+  else
+    our_ref_ip.family = IPADDR_UNSPEC;
   our_ref_time = *ref_time;
   our_offset = offset;
   our_root_delay = root_delay;
   our_root_dispersion = root_dispersion;
 
+  update_leap_status(leap);
+
   /* Eliminate updates that are based on totally unreliable frequency
      information */
 
@@ -386,7 +497,7 @@ REF_SetReference(int stratum,
   } else {
 
 #if 0    
-    LOG(LOGS_INFO, LOGF_Reference, "Skew %f too large to track, offset=%f\n", skew, our_offset);
+    LOG(LOGS_INFO, LOGF_Reference, "Skew %f too large to track, offset=%f", skew, our_offset);
 #endif
     maybe_log_offset(our_offset);
     LCL_AccumulateOffset(our_offset);
@@ -394,27 +505,16 @@ REF_SetReference(int stratum,
     our_residual_freq = frequency;
   }
 
+  maybe_make_step();
+
   abs_freq_ppm = LCL_ReadAbsoluteFrequency();
 
-  if (logfile) {
-
-    if (((logwrites++) % 32) == 0) {
-      fprintf(logfile,
-              "====================================================================\n"
-              " Date (UTC) Time    IP Address   St   Freq ppm   Skew ppm     Offset\n"
-              "====================================================================\n");
-    }
-          
-    fprintf(logfile, "%s %-15s %2d %10.3f %10.3f %10.3e\n",
-            UTI_TimeToLogForm(ref_time->tv_sec),
-            UTI_IPToDottedQuad(our_ref_id),
+  write_log(ref_time,
+            our_ref_ip.family != IPADDR_UNSPEC ? UTI_IPToString(&our_ref_ip) : UTI_RefidToString(our_ref_id),
             our_stratum,
             abs_freq_ppm,
             1.0e6*our_skew,
             our_offset);
-    
-    fflush(logfile);
-  }
 
   if (drift_file) {
     update_drift_file(abs_freq_ppm, our_skew);
@@ -438,7 +538,6 @@ REF_SetManualReference
  double skew
 )
 {
-  int millisecond;
   double abs_freq_ppm;
 
   /* We are not synchronised to an external source, as such.  This is
@@ -451,23 +550,17 @@ REF_SetManualReference
 
   maybe_log_offset(offset);
   LCL_AccumulateFrequencyAndOffset(frequency, offset);
+  maybe_make_step();
 
   abs_freq_ppm = LCL_ReadAbsoluteFrequency();
 
-  if (logfile) {
-    millisecond = ref_time->tv_usec / 1000;
-
-    fprintf(logfile, "%5s %-15s %2d %10.3f %10.3f %10.3e\n",
-            UTI_TimeToLogForm(ref_time->tv_sec),
+  write_log(ref_time,
             "127.127.1.1",
             our_stratum,
             abs_freq_ppm,
             1.0e6*our_skew,
             our_offset);
 
-    fflush(logfile);
-  }
-
   if (drift_file) {
     update_drift_file(abs_freq_ppm, our_skew);
   }
@@ -479,28 +572,23 @@ void
 REF_SetUnsynchronised(void)
 {
   /* Variables required for logging to statistics log */
-  int millisecond;
   struct timeval now;
   double local_clock_err;
 
   assert(initialised);
 
-  if (logfile) {
-    LCL_ReadCookedTime(&now, &local_clock_err);
+  LCL_ReadCookedTime(&now, &local_clock_err);
 
-    millisecond = now.tv_usec / 1000;
-
-    fprintf(logfile, "%s %-15s  0 %10.3f %10.3f %10.3e\n",
-            UTI_TimeToLogForm(now.tv_sec),
+  write_log(&now,
             "0.0.0.0",
+            0,
             LCL_ReadAbsoluteFrequency(),
             1.0e6*our_skew,
             0.0);
-    
-    fflush(logfile);
-  }
 
   are_we_synchronised = 0;
+
+  update_leap_status(LEAP_Unsynchronised);
 }
 
 /* ================================================== */
@@ -597,7 +685,7 @@ REF_ModifyMaxupdateskew(double new_max_update_skew)
 {
   max_update_skew = new_max_update_skew * 1.0e-6;
 #if 0
-  LOG(LOGS_INFO, LOGF_Reference, "New max update skew = %.3fppm\n", new_max_update_skew);
+  LOG(LOGS_INFO, LOGF_Reference, "New max update skew = %.3fppm", new_max_update_skew);
 #endif
 }
 
@@ -620,6 +708,14 @@ REF_DisableLocal(void)
 
 /* ================================================== */
 
+int
+REF_IsLocalActive(void)
+{
+  return !are_we_synchronised && enable_local_stratum;
+}
+
+/* ================================================== */
+
 void
 REF_GetTrackingReport(RPT_TrackingReport *rep)
 {
@@ -638,9 +734,10 @@ REF_GetTrackingReport(RPT_TrackingReport *rep)
     extra_dispersion = (our_skew + fabs(our_residual_freq)) * elapsed;
     
     rep->ref_id = our_ref_id;
+    rep->ip_addr = our_ref_ip;
     rep->stratum = our_stratum;
     rep->ref_time = our_ref_time;
-    UTI_DoubleToTimeval(correction, &rep->current_correction);
+    rep->current_correction = correction;
     rep->freq_ppm = LCL_ReadAbsoluteFrequency();
     rep->resid_freq_ppm = 1.0e6 * our_residual_freq;
     rep->skew_ppm = 1.0e6 * our_skew;
@@ -650,9 +747,10 @@ REF_GetTrackingReport(RPT_TrackingReport *rep)
   } else if (enable_local_stratum) {
 
     rep->ref_id = LOCAL_REFERENCE_ID;
+    rep->ip_addr.family = IPADDR_UNSPEC;
     rep->stratum = local_stratum;
     rep->ref_time = now_cooked;
-    UTI_DoubleToTimeval(correction, &rep->current_correction);
+    rep->current_correction = correction;
     rep->freq_ppm = LCL_ReadAbsoluteFrequency();
     rep->resid_freq_ppm = 0.0;
     rep->skew_ppm = 0.0;
@@ -662,10 +760,11 @@ REF_GetTrackingReport(RPT_TrackingReport *rep)
   } else {
 
     rep->ref_id = 0UL;
+    rep->ip_addr.family = IPADDR_UNSPEC;
     rep->stratum = 0;
     rep->ref_time.tv_sec = 0;
     rep->ref_time.tv_usec = 0;
-    UTI_DoubleToTimeval(correction, &rep->current_correction);
+    rep->current_correction = correction;
     rep->freq_ppm = LCL_ReadAbsoluteFrequency();
     rep->resid_freq_ppm = 0.0;
     rep->skew_ppm = 0.0;
@@ -684,7 +783,7 @@ REF_CycleLogFile(void)
     fclose(logfile);
     logfile = fopen(logfilename, "a");
     if (!logfile) {
-      LOG(LOGS_WARN, LOGF_Reference, "Could not reopen logfile %s\n", logfilename);
+      LOG(LOGS_WARN, LOGF_Reference, "Could not reopen logfile %s", logfilename);
     }
     logwrites = 0;
   }
