@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/reference.h,v 1.11 1999/04/19 20:27:29 richard Exp $
+  $Header: /cvs/src/chrony/reference.h,v 1.13 2002/02/28 23:27:12 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2002
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -97,6 +110,7 @@ extern void REF_SetReference
  int stratum,
  NTP_Leap leap,
  unsigned long ref_id,
+ IPAddr *ref_ip,
  struct timeval *ref_time,
  double offset,
  double frequency,
@@ -126,6 +140,7 @@ extern void REF_ModifyMaxupdateskew(double new_max_update_skew);
 
 extern void REF_EnableLocal(int stratum);
 extern void REF_DisableLocal(void);
+extern int REF_IsLocalActive(void);
 
 extern void REF_GetTrackingReport(RPT_TrackingReport *rep);
 
