@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/sched.c,v 1.13 2000/06/12 21:22:49 richard Exp $
+  $Header: /cvs/src/chrony/sched.c,v 1.17 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -58,6 +71,9 @@ typedef struct {
 
 static FileHandlerEntry file_handlers[FD_SET_SIZE];
 
+/* Last timestamp when a file descriptor became readable */
+static struct timeval last_fdready;
+
 /* ================================================== */
 
 /* Variables to handler the timer queue */
@@ -212,6 +228,14 @@ SCH_RemoveInputFileHandler(int fd)
 
 /* ================================================== */
 
+void
+SCH_GetFileReadyTime(struct timeval *tv)
+{
+  *tv = last_fdready;
+}
+
+/* ================================================== */
+
 #define TQE_ALLOC_QUANTUM 32
 
 static TimerQueueEntry *
@@ -416,7 +440,7 @@ dispatch_timeouts(struct timeval *now) {
   TimerQueueEntry *ptr;
   int n_done = 0;
 
-  while ((n_timer_queue_entries > 0) &&
+  if ((n_timer_queue_entries > 0) &&
          (UTI_CompareTimevals(now, &(timer_queue.next->tv)) >= 0)) {
     ptr = timer_queue.next;
 
@@ -497,10 +521,11 @@ handle_slew(struct timeval *raw,
 void
 SCH_MainLoop(void)
 {
-  fd_set rd, wr, ex;
+  fd_set rd;
   int status;
   struct timeval tv, *ptv;
   struct timeval now;
+  double err;
 
   if (!initialised) {
     CROAK("Should be initialised");
@@ -511,11 +536,6 @@ SCH_MainLoop(void)
     /* Copy current set of read file descriptors */
     memcpy((void *) &rd, (void *) &read_fds, sizeof(fd_set));
     
-    /* Blank the write and exception descriptors - we aren't very
-       interested */
-    FD_ZERO(&wr);
-    FD_ZERO(&ex);
-
     /* Try to dispatch any timeouts that have already gone by, and
        keep going until all are done.  (The earlier ones may take so
        long to do that the later ones come around by the time they are
@@ -539,16 +559,18 @@ SCH_MainLoop(void)
        timeout set, this is clearly ridiculous, so stop the run */
 
     if (!ptv && (n_read_fds == 0)) {
-      LOG_FATAL(LOGF_Scheduler, "No descriptors or timeout to wait for\n");
+      LOG_FATAL(LOGF_Scheduler, "No descriptors or timeout to wait for");
     }
 
-    status = select(one_highest_fd, &rd, &wr, &ex, ptv);
+    status = select(one_highest_fd, &rd, NULL, NULL, ptv);
 
     if (status < 0) {
-      CROAK("Status < 0 after select");
+      if (!need_to_exit)
+        CROAK("Status < 0 after select");
     } else if (status > 0) {
       /* A file descriptor is ready to read */
 
+      LCL_ReadCookedTime(&last_fdready, &err);
       dispatch_filehandlers(status, &rd);
 
     } else {
