@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/sched.h,v 1.7 1999/04/19 20:27:29 richard Exp $
+  $Header: /cvs/src/chrony/sched.h,v 1.10 2002/02/28 23:27:14 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2002
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -25,6 +38,7 @@ typedef unsigned long SCH_TimeoutID;
 typedef unsigned long SCH_TimeoutClass;
 static const SCH_TimeoutClass SCH_ReservedTimeoutValue = 0;
 static const SCH_TimeoutClass SCH_NtpSamplingClass = 1;
+static const SCH_TimeoutClass SCH_NtpBroadcastClass = 2;
 
 typedef void* SCH_ArbitraryArgument;
 typedef void (*SCH_FileHandler)(SCH_ArbitraryArgument);
@@ -46,6 +60,10 @@ extern void SCH_AddInputFileHandler
 );
 extern void SCH_RemoveInputFileHandler(int fd);
 
+/* Get the time (cooked) when file descriptor became ready, intended for use
+   in file handlers */
+extern void SCH_GetFileReadyTime(struct timeval *tv);
+
 /* This queues a timeout to elapse at a given (raw) local time */
 extern SCH_TimeoutID SCH_AddTimeout(struct timeval *tv, SCH_TimeoutHandler, SCH_ArbitraryArgument);
 
